@@ -22,7 +22,7 @@ extern "C" BinaryenModuleRef BinaryenModuleSafeRead(const char* input, size_t in
     auto* wasm = new Module;
     vector<char> buffer(input, input + inputSize);
     try {
-        WasmBinaryBuilder parser(*wasm, buffer, false);
+        WasmBinaryBuilder parser(*wasm, buffer);
         parser.read();
     } catch (ParseException const&) {
         // FIXME: support passing back the exception text
@@ -89,3 +89,14 @@ extern "C" void BinaryenModuleRunPassesWithSettings(
   }
   passRunner.run();
 }
+
+// NOTE: this is based on BinaryenModuleValidate from binaryen-c.cpp
+extern "C" int BinaryenModuleSafeValidate(BinaryenModuleRef module) {
+  Module* wasm = (Module*)module;
+  auto features = wasm->features;
+  // TODO(tlively): Add C API for managing features
+  wasm->features = FeatureSet::All;
+  auto ret = WasmValidator().validate(*wasm) ? 1 : 0;
+  wasm->features = features;
+  return ret;
+}
