@@ -18,14 +18,14 @@
 // Removes obviously unneeded code
 //
 
-#include <wasm.h>
-#include <pass.h>
-#include <wasm-builder.h>
 #include <ir/block-utils.h>
 #include <ir/effects.h>
 #include <ir/literal-utils.h>
 #include <ir/type-updating.h>
 #include <ir/utils.h>
+#include <pass.h>
+#include <wasm-builder.h>
+#include <wasm.h>
 
 namespace wasm {
 
@@ -49,15 +49,19 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
     walk(func->body);
   }
 
-  // Returns nullptr if curr is dead, curr if it must stay as is, or another node if it can be replaced.
-  // Takes into account:
+  // Returns nullptr if curr is dead, curr if it must stay as is, or another
+  // node if it can be replaced. Takes into account:
   //  * The result may be used or unused.
   //  * The type may or may not matter (a drop can drop anything, for example).
   Expression* optimize(Expression* curr, bool resultUsed, bool typeMatters) {
+    FeatureSet features = getModule()->features;
     auto type = curr->type;
     // An unreachable node must not be changed.
-    if (type == unreachable) return curr;
-    // We iterate on possible replacements. If a replacement changes the type, stop and go back.
+    if (type == Type::unreachable) {
+      return curr;
+    }
+    // We iterate on possible replacements. If a replacement changes the type,
+    // stop and go back.
     auto* prev = curr;
     while (1) {
       if (typeMatters && curr->type != type) {
@@ -65,36 +69,49 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
       }
       prev = curr;
       switch (curr->_id) {
-        case Expression::Id::NopId: return nullptr; // never needed
+        case Expression::Id::NopId:
+          return nullptr; // never needed
 
-        case Expression::Id::BlockId: return curr; // not always needed, but handled in visitBlock()
-        case Expression::Id::IfId: return curr; // not always needed, but handled in visitIf()
-        case Expression::Id::LoopId: return curr; // not always needed, but handled in visitLoop()
-        case Expression::Id::DropId: return curr; // not always needed, but handled in visitDrop()
+        case Expression::Id::BlockId:
+          return curr; // not always needed, but handled in visitBlock()
+        case Expression::Id::IfId:
+          return curr; // not always needed, but handled in visitIf()
+        case Expression::Id::LoopId:
+          return curr; // not always needed, but handled in visitLoop()
+        case Expression::Id::DropId:
+          return curr; // not always needed, but handled in visitDrop()
 
         case Expression::Id::BreakId:
         case Expression::Id::SwitchId:
         case Expression::Id::CallId:
         case Expression::Id::CallIndirectId:
-        case Expression::Id::SetLocalId:
+        case Expression::Id::LocalSetId:
         case Expression::Id::StoreId:
         case Expression::Id::ReturnId:
-        case Expression::Id::SetGlobalId:
+        case Expression::Id::GlobalSetId:
         case Expression::Id::HostId:
-        case Expression::Id::UnreachableId: return curr; // always needed
+        case Expression::Id::UnreachableId:
+          return curr; // always needed
 
         case Expression::Id::LoadId: {
           // it is ok to remove a load if the result is not used, and it has no
-          // side effects (the load itself may trap, if we are not ignoring such things)
-          if (!resultUsed && !EffectAnalyzer(getPassOptions(), curr).hasSideEffects()) {
-            return curr->cast<Load>()->ptr;
+          // side effects (the load itself may trap, if we are not ignoring such
+          // things)
+          auto* load = curr->cast<Load>();
+          if (!resultUsed && !EffectAnalyzer(getPassOptions(), features, curr)
+                                .hasSideEffects()) {
+            if (!typeMatters || load->ptr->type == type) {
+              return load->ptr;
+            }
           }
           return curr;
         }
         case Expression::Id::ConstId:
-        case Expression::Id::GetLocalId:
-        case Expression::Id::GetGlobalId: {
-          if (!resultUsed) return nullptr;
+        case Expression::Id::LocalGetId:
+        case Expression::Id::GlobalGetId: {
+          if (!resultUsed) {
+            return nullptr;
+          }
           return curr;
         }
 
@@ -104,35 +121,40 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
           if (resultUsed) {
             return curr; // used, keep it
           }
-          // for unary, binary, and select, we need to check their arguments for side effects,
-          // as well as the node itself, as some unaries and binaries have implicit traps
+          // for unary, binary, and select, we need to check their arguments for
+          // side effects, as well as the node itself, as some unaries and
+          // binaries have implicit traps
           if (auto* unary = curr->dynCast<Unary>()) {
-            EffectAnalyzer tester(getPassOptions());
+            EffectAnalyzer tester(getPassOptions(), features);
             tester.visitUnary(unary);
             if (tester.hasSideEffects()) {
               return curr;
             }
-            if (EffectAnalyzer(getPassOptions(), unary->value).hasSideEffects()) {
+            if (EffectAnalyzer(getPassOptions(), features, unary->value)
+                  .hasSideEffects()) {
               curr = unary->value;
               continue;
             } else {
               return nullptr;
             }
           } else if (auto* binary = curr->dynCast<Binary>()) {
-            EffectAnalyzer tester(getPassOptions());
+            EffectAnalyzer tester(getPassOptions(), features);
             tester.visitBinary(binary);
             if (tester.hasSideEffects()) {
               return curr;
             }
-            if (EffectAnalyzer(getPassOptions(), binary->left).hasSideEffects()) {
-              if (EffectAnalyzer(getPassOptions(), binary->right).hasSideEffects()) {
+            if (EffectAnalyzer(getPassOptions(), features, binary->left)
+                  .hasSideEffects()) {
+              if (EffectAnalyzer(getPassOptions(), features, binary->right)
+                    .hasSideEffects()) {
                 return curr; // leave them
               } else {
                 curr = binary->left;
                 continue;
               }
             } else {
-              if (EffectAnalyzer(getPassOptions(), binary->right).hasSideEffects()) {
+              if (EffectAnalyzer(getPassOptions(), features, binary->right)
+                    .hasSideEffects()) {
                 curr = binary->right;
                 continue;
               } else {
@@ -140,13 +162,18 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
               }
             }
           } else {
-            // TODO: if two have side effects, we could replace the select with say an add?
+            // TODO: if two have side effects, we could replace the select with
+            // say an add?
             auto* select = curr->cast<Select>();
-            if (EffectAnalyzer(getPassOptions(), select->ifTrue).hasSideEffects()) {
-              if (EffectAnalyzer(getPassOptions(), select->ifFalse).hasSideEffects()) {
+            if (EffectAnalyzer(getPassOptions(), features, select->ifTrue)
+                  .hasSideEffects()) {
+              if (EffectAnalyzer(getPassOptions(), features, select->ifFalse)
+                    .hasSideEffects()) {
                 return curr; // leave them
               } else {
-                if (EffectAnalyzer(getPassOptions(), select->condition).hasSideEffects()) {
+                if (EffectAnalyzer(
+                      getPassOptions(), features, select->condition)
+                      .hasSideEffects()) {
                   return curr; // leave them
                 } else {
                   curr = select->ifTrue;
@@ -154,15 +181,20 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
                 }
               }
             } else {
-              if (EffectAnalyzer(getPassOptions(), select->ifFalse).hasSideEffects()) {
-                if (EffectAnalyzer(getPassOptions(), select->condition).hasSideEffects()) {
+              if (EffectAnalyzer(getPassOptions(), features, select->ifFalse)
+                    .hasSideEffects()) {
+                if (EffectAnalyzer(
+                      getPassOptions(), features, select->condition)
+                      .hasSideEffects()) {
                   return curr; // leave them
                 } else {
                   curr = select->ifFalse;
                   continue;
                 }
               } else {
-                if (EffectAnalyzer(getPassOptions(), select->condition).hasSideEffects()) {
+                if (EffectAnalyzer(
+                      getPassOptions(), features, select->condition)
+                      .hasSideEffects()) {
                   curr = select->condition;
                   continue;
                 } else {
@@ -173,12 +205,13 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
           }
         }
 
-        default: return curr; // assume needed
+        default:
+          return curr; // assume needed
       }
     }
   }
 
-  void visitBlock(Block *curr) {
+  void visitBlock(Block* curr) {
     // compress out nops and other dead code
     int skip = 0;
     auto& list = curr->list;
@@ -186,18 +219,20 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
     for (size_t z = 0; z < size; z++) {
       auto* child = list[z];
       // The last element may be used.
-      bool used = z == size - 1 &&
-                  isConcreteType(curr->type) &&
-                  ExpressionAnalyzer::isResultUsed(expressionStack, getFunction());
+      bool used =
+        z == size - 1 && curr->type.isConcrete() &&
+        ExpressionAnalyzer::isResultUsed(expressionStack, getFunction());
       auto* optimized = optimize(child, used, true);
       if (!optimized) {
-        if (isConcreteType(child->type)) {
-          // We can't just skip a final concrete element, even if it isn't used. Instead,
-          // replace it with something that's easy to optimize out (for example, code-folding
-          // can merge out identical zeros at the end of if arms).
+        if (child->type.isConcrete()) {
+          // We can't just skip a final concrete element, even if it isn't used.
+          // Instead, replace it with something that's easy to optimize out (for
+          // example, code-folding can merge out identical zeros at the end of
+          // if arms).
           optimized = LiteralUtils::makeZero(child->type, *getModule());
-        } else if (child->type == unreachable) {
-          // Don't try to optimize out an unreachable child (dce can do that properly).
+        } else if (child->type == Type::unreachable) {
+          // Don't try to optimize out an unreachable child (dce can do that
+          // properly).
           optimized = child;
         }
       }
@@ -214,7 +249,7 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
           list[z] = nullptr;
         }
         // if this is unreachable, the rest is dead code
-        if (list[z - skip]->type == unreachable && z < size - 1) {
+        if (list[z - skip]->type == Type::unreachable && z < size - 1) {
           for (Index i = z - skip + 1; i < list.size(); i++) {
             auto* remove = list[i];
             if (remove) {
@@ -232,7 +267,8 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
       list.resize(size - skip);
       typeUpdater.maybeUpdateTypeToUnreachable(curr);
     }
-    // the block may now be a trivial one that we can get rid of and just leave its contents
+    // the block may now be a trivial one that we can get rid of and just leave
+    // its contents
     replaceCurrent(BlockUtils::simplifyToContents(curr, this));
   }
 
@@ -260,7 +296,7 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
       return;
     }
     // if the condition is unreachable, just return it
-    if (curr->condition->type == unreachable) {
+    if (curr->condition->type == Type::unreachable) {
       typeUpdater.noteRecursiveRemoval(curr->ifTrue);
       if (curr->ifFalse) {
         typeUpdater.noteRecursiveRemoval(curr->ifFalse);
@@ -275,9 +311,11 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
       } else if (curr->ifTrue->is<Nop>()) {
         curr->ifTrue = curr->ifFalse;
         curr->ifFalse = nullptr;
-        curr->condition = Builder(*getModule()).makeUnary(EqZInt32, curr->condition);
+        curr->condition =
+          Builder(*getModule()).makeUnary(EqZInt32, curr->condition);
       } else if (curr->ifTrue->is<Drop>() && curr->ifFalse->is<Drop>()) {
-        // instead of dropping both sides, drop the if, if they are the same type
+        // instead of dropping both sides, drop the if, if they are the same
+        // type
         auto* left = curr->ifTrue->cast<Drop>()->value;
         auto* right = curr->ifFalse->cast<Drop>()->value;
         if (left->type == right->type) {
@@ -297,7 +335,9 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
   }
 
   void visitLoop(Loop* curr) {
-    if (curr->body->is<Nop>()) ExpressionManipulator::nop(curr);
+    if (curr->body->is<Nop>()) {
+      ExpressionManipulator::nop(curr);
+    }
   }
 
   void visitDrop(Drop* curr) {
@@ -308,37 +348,38 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
       return;
     }
     // a drop of a tee is a set
-    if (auto* set = curr->value->dynCast<SetLocal>()) {
+    if (auto* set = curr->value->dynCast<LocalSet>()) {
       assert(set->isTee());
-      set->setTee(false);
+      set->makeSet();
       replaceCurrent(set);
       return;
     }
-    // if we are dropping a block's return value, we might be able to remove it entirely
+    // if we are dropping a block's return value, we might be able to remove it
+    // entirely
     if (auto* block = curr->value->dynCast<Block>()) {
       auto* last = block->list.back();
       // note that the last element may be concrete but not the block, if the
-      // block has an unreachable element in the middle, making the block unreachable
-      // despite later elements and in particular the last
-      if (isConcreteType(last->type) && block->type == last->type) {
+      // block has an unreachable element in the middle, making the block
+      // unreachable despite later elements and in particular the last
+      if (last->type.isConcrete() && block->type == last->type) {
         last = optimize(last, false, false);
         if (!last) {
           // we may be able to remove this, if there are no brs
           bool canPop = true;
           if (block->name.is()) {
             BranchUtils::BranchSeeker seeker(block->name);
-            seeker.named = true;
             Expression* temp = block;
             seeker.walk(temp);
-            if (seeker.found && seeker.valueType != none) {
+            if (seeker.found && seeker.valueType != Type::none) {
               canPop = false;
             }
           }
           if (canPop) {
             block->list.back() = last;
             block->list.pop_back();
-            block->type = none;
-            // we don't need the drop anymore, let's see what we have left in the block
+            block->type = Type::none;
+            // we don't need the drop anymore, let's see what we have left in
+            // the block
             if (block->list.size() > 1) {
               replaceCurrent(block);
             } else if (block->list.size() == 1) {
@@ -351,40 +392,44 @@ struct Vacuum : public WalkerPass<ExpressionStackWalker<Vacuum>> {
         }
       }
     }
-    // sink a drop into an arm of an if-else if the other arm ends in an unreachable, as it if is a branch, this can make that branch optimizable and more vaccuming possible
+    // sink a drop into an arm of an if-else if the other arm ends in an
+    // unreachable, as it if is a branch, this can make that branch optimizable
+    // and more vaccuming possible
     auto* iff = curr->value->dynCast<If>();
-    if (iff && iff->ifFalse && isConcreteType(iff->type)) {
+    if (iff && iff->ifFalse && iff->type.isConcrete()) {
       // reuse the drop in both cases
-      if (iff->ifTrue->type == unreachable && isConcreteType(iff->ifFalse->type)) {
+      if (iff->ifTrue->type == Type::unreachable &&
+          iff->ifFalse->type.isConcrete()) {
         curr->value = iff->ifFalse;
         iff->ifFalse = curr;
-        iff->type = none;
+        iff->type = Type::none;
         replaceCurrent(iff);
-      } else if (iff->ifFalse->type == unreachable && isConcreteType(iff->ifTrue->type)) {
+      } else if (iff->ifFalse->type == Type::unreachable &&
+                 iff->ifTrue->type.isConcrete()) {
         curr->value = iff->ifTrue;
         iff->ifTrue = curr;
-        iff->type = none;
+        iff->type = Type::none;
         replaceCurrent(iff);
       }
     }
   }
 
   void visitFunction(Function* curr) {
-    auto* optimized = optimize(curr->body, curr->result != none, true);
+    auto* optimized =
+      optimize(curr->body, curr->sig.results != Type::none, true);
     if (optimized) {
       curr->body = optimized;
     } else {
       ExpressionManipulator::nop(curr->body);
     }
-    if (curr->result == none && !EffectAnalyzer(getPassOptions(), curr->body).hasSideEffects()) {
+    if (curr->sig.results == Type::none &&
+        !EffectAnalyzer(getPassOptions(), getModule()->features, curr->body)
+           .hasSideEffects()) {
       ExpressionManipulator::nop(curr->body);
     }
   }
 };
 
-Pass *createVacuumPass() {
-  return new Vacuum();
-}
+Pass* createVacuumPass() { return new Vacuum(); }
 
 } // namespace wasm
-
